module ClassLiteral (ClassLiteral(..), parseClassLiteral ) where

import qualified Data.List as List
import Numeric.Natural (Natural)

import PrimitiveTypes (PrimitiveType, primitiveType)
import JavaLanguage (m_reserved, m_lexeme, m_identifier)

import Text.Parsec (choice, string', many, sepBy, char, manyTill, lookAhead)
import Text.Parsec.String (Parser)

data ClassLiteral = Void 
        | Primitive PrimitiveType Natural -- type, array dimension
        | Type String Natural -- type, array dimension
        deriving Show

parseClassLiteral :: Parser ClassLiteral
parseClassLiteral = choice 
        [ m_reserved "void" *>  return Void
        , primitiveType     >>= parseArrayDims . Primitive
        , typeName          >>= parseArrayDims . Type
        ] <* string' ".class"

typeName :: Parser String
typeName = do
        start <- m_identifier
        paths <- manyTill (char '.' *> m_identifier) (choice [ lookAhead $ string' ".class" , lookAhead $ string' "[]" ])
        return . List.intercalate "." $ (start:paths)

parseArrayDims :: (Natural -> ClassLiteral) -> Parser ClassLiteral
parseArrayDims f = m_lexeme $ many (string' "[]") >>= return . f . fromInteger . toInteger . List.length
